/*
 *	fcondb.h	F-CON Monitor packet contents database
 *				Copyright(C)2003 Kashi Electronics Designs,
 *				All Rights Reserved.
 *	Ver 1.0	Sep.8/2003 first release
 *
 *   This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *   This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 */
#include "fcon.h"

const char	sUnit[][4] = {
		"rpm", "mV ", "Kg ", "%  ", "CA ",
		"-BD", "uS ", "mS ", "Km ", "   ",
		"\x03  ", "mHg", "mV", "%  ", "   ",
		"   ", "   ", "V  ", "lpm"
};

//
// moniter packet definition for F-CON Vpro(gold)
//
//
const struct	tsMonEnt	MonEntVP5[] = {
	{ L1_ESPED, 0x00, "ESPED", T_UI , U_RPM },
	{ L1_MAFS,  0x02, "MAFS",  T_UI , U_PCT },
	{ L1_THR,   0x04, "THR",   T_UI , U_PCT },
	{ L1_WTEMP, 0x06, "WTEMP", T_TIN, U_C   },
	{ L1_ITEMP, 0x08, "ITEMP", T_TIN, U_C   },
	{ L1_IPRES, 0x0a, "IPRES", T_UI , U_MHG },
	{ L1_AF,    0x0c, "AF",    T_UI , U_NA  },
	{ L1_ETEMP, 0x0e, "ETEMP", T_UI , U_C   },
	{ L1_FPRES, 0x10, "FPRES", T_UI , U_KG  },
	{ L1_VBATT, 0x12, "VBATT", T_I1K, U_V   },
	{ L1_VSPED, 0x14, "VSPED", T_UI , U_KM  },
	{ L1_SW,    0x16, "SW",    T_BOL, U_NA  },
	{ L1_DESPD, 0x18, "DESPD", T_INT, U_PCT },
	{ L1_DTHR,  0x1a, "DTHR",  T_INT, U_PCT },
	{ L1_KNOCK, 0x1c, "KNOCK", T_UI , U_NA  },
	{ L1_FUEL1, 0x20, "FUEL1", T_UI , U_US  },
	{ L1_IGNP1, 0x22, "IGNP1", T_INT, U_BD  },
	{ L1_FUEL2, 0x24, "FUEL2", T_UI , U_US  },
	{ L1_IGNP2, 0x26, "IGNP2", T_INT, U_BD  },
	{ L1_ISCV,  0x28, "ISCV",  T_DEG, U_PCT },
	{ L1_TWIN,  0x2a, "TWIN",  T_DEG, U_PCT },
	{ L1_FCTOT, 0x32, "FCTOT", T_DEG, U_NA  },
	{ L1_FCMAP, 0x34, "FCMAP", T_DEG, U_NA  },
	{ L1_FCTHR, 0x36, "FCTHR", T_DEG, U_NA  },
	{ L1_FCVSP, 0x38, "FCVSP", T_DEG, U_NA  },
	{ L1_FCWTP, 0x3a, "FCWTP", T_DEG, U_NA  },
	{ L1_FCITP, 0x3c, "FCITP", T_DEG, U_NA  },
	{ L1_FCETP, 0x3e, "FCETP", T_DEG, U_NA  },
	{ L1_FCFPR, 0x40, "FCFPR", T_DEG, U_NA  },
	{ L1_FCVOL, 0x42, "FCVOL", T_DEG, U_NA  },
	{ L1_FCLOD, 0x44, "FCLOD", T_DEG, U_NA  },
	{ L1_FCACC, 0x46, "FCACC", T_DEG, U_NA  },
	{ L1_FCDEC, 0x48, "FCDEC", T_DEG, U_NA  },
	{ L1_FCSTA, 0x4a, "FCSTA", T_DEG, U_NA  },
	{ L1_FCAIJ, 0x4c, "FCAIJ", T_DEG, U_NA  },
	{ L1_FCINJ, 0x4e, "FCINJ", T_DEG, U_NA  },
	{ L1_FCIDL, 0x50, "FCIDL", T_DEG, U_NA  },
	{ L1_IRMAP, 0x62, "IRMAP", T_NCH, U_NA  },
	{ L1_IRTHR, 0x63, "IRTHR", T_NCH, U_NA  },
	{ L1_IRVSP, 0x64, "IRVSP", T_NCH, U_NA  },
	{ L1_IRWTP, 0x65, "IRWTP", T_NCH, U_NA  },
	{ L1_IRITP, 0x66, "IRITP", T_NCH, U_NA  },
	{ L1_IRETP, 0x67, "IRETP", T_NCH, U_NA  },
	{ L1_IRFPR, 0x68, "IRFPR", T_NCH, U_NA  },
	{ L1_IRVOL, 0x69, "IRVOL", T_NCH, U_NA  },
	{ L1_IRLOD, 0x6a, "IRLOD", T_NCH, U_NA  },
	{ L1_IRACC, 0x6b, "IRACC", T_NCH, U_NA  },
	{ L1_IRDEC, 0x6c, "IRDEC", T_NCH, U_NA  },
	{ L1_IRSTA, 0x6d, "IRSTA", T_NCH, U_NA  },
	{ L1_IRIDL, 0x6e, "IRIDL", T_NCH, U_NA  },
	{ L1_IGDIS, 0x6f, "IGDIS", T_NCH, U_NA  },
	{ L1_O2ACT, 0x78, "O2ACT", T_BOL, U_NA  },
	{ L1_ACSW,  0x79, "ACSW",  T_BOL, U_NA  },
	{ L1_LDSW1, 0x7a, "LDSW1", T_BOL, U_NA  },
	{ L1_LDSW2, 0x7b, "LDSW2", T_BOL, U_NA  },
	{ L1_NTRL,  0x7c, "NTRL",  T_BOL, U_NA  }
};

//
// moniter packet definition for F-CON S
//
//
const struct	tsMonEnt	MonEntFCS[] = {
	{ L2_ESPED, 0x20, "ESPED", T_UI , U_RPM },
	{ L2_IPRES, 0x22, "IPRES", T_35I, U_KG  },
	{ L2_THR,   0x24, "THR",   T_DEG, U_PCT },
	{ L2_DTHR,  0x26, "DTHR",  T_DEG, U_PCT },
	{ L2_MAFS1, 0x28, "MAFS1", T_UI , U_MV  },
	{ L2_MAFS2, 0x2a, "MAFS2", T_UI , U_MV  },
	{ L2_VSPED, 0x2e, "VSPED", T_UI , U_KM  },
	{ L2_AF,    0x30, "AF",    T_100, U_NA  },
	{ L2_F1IN,  0x32, "F1IN",  T_INT, U_US  },
	{ L2_F2IN,  0x34, "F2IN",  T_INT, U_NA  },
	{ L2_F1OUT, 0x38, "F1OUT", T_INT, U_US  },
	{ L2_F2OUT, 0x3a, "F2OUT", T_INT, U_US  },
	{ L2_AICO,  0x3e, "AICO",  T_INT, U_US  },
	{ L2_AICD,  0x42, "AICD",  T_INT, U_US  },
	{ L2_VBATT, 0x44, "VBATT", T_100, U_V   },
	{ L2_VOPT1, 0x46, "VOPT1", T_INT, U_MV  },
	{ L2_VOPT2, 0x48, "VOPT2", T_INT, U_US  },
	{ L2_WTEMP, 0x4a, "WTEMP", T_TIN, U_C   },
	{ L2_ITEMP, 0x4c, "ITEMP", T_TIN, U_C   },
	{ L2_FTOT,  0x4e, "FTOT",  T_CHR, U_NA  },
	{ L2_ITOT,  0x4f, "ITOT",  T_05C, U_NA  },
	{ L2_ATOT,  0x50, "ATOT",  T_CHR, U_NA  },
	{ L2_FCOR1, 0x51, "FCOR1", T_CHR, U_NA  },
	{ L2_FCOR2, 0x52, "FCOR2", T_CHR, U_NA  },
	{ L2_FCWTP, 0x53, "FCWTP", T_CHR, U_NA  },
	{ L2_FCITP, 0x54, "FCITP", T_CHR, U_NA  },
	{ L2_FCGCC, 0x55, "FCGCC", T_CHR, U_NA  },
	{ L2_FCVPC, 0x56, "FCVPC", T_CHR, U_NA  },
	{ L2_FCSBC, 0x57, "FCSBC", T_CHR, U_NA  },
	{ L2_FCACC, 0x58, "FCACC", T_CHR, U_NA  },
	{ L2_IR1,   0x59, "IR1",   T_05C, U_NA  },
	{ L2_IR2,   0x5a, "IR2",   T_05C, U_NA  },
	{ L2_IRWTP, 0x5b, "IRWTP", T_05C, U_NA  },
	{ L2_IRITP, 0x5c, "IRITP", T_05C, U_NA  },
	{ L2_IRGCC, 0x5d, "IRGCC", T_05C, U_NA  },
	{ L2_IRVPC, 0x5e, "IRVPC", T_05C, U_NA  },
	{ L2_IRSBC, 0x5f, "IRSBC", T_05C, U_NA  },
	{ L2_IRACC, 0x60, "IRACC", T_05C, U_NA  },
	{ L2_AWTMP, 0x61, "AWTMP", T_CHR, U_NA  },
	{ L2_AITMP, 0x62, "AITMP", T_CHR, U_NA  },
	{ L2_TSBC,  0x63, "TSBC",  T_CHR, U_NA  }
};

//
// moniter packet definition for F-CON Vpro(silver)
//
//
const struct	tsMonEnt	MonEntVP2[] = {
	{ L3_ESPED, 0x50, "ESPED", T_UI , U_RPM },
	{ L3_MAFS , 0x52, "MAFS" , T_UI , U_L   },
	{ L3_THR  , 0x54, "THR"  , T_UI , U_PCT },
	{ L3_WTEMP, 0x56, "WTEMP", T_IM5, U_C   },
	{ L3_ITEMP, 0x58, "ITEMP", T_IM5, U_MV  },
	{ L3_IPRES, 0x5a, "IPRES", T_35I, U_KG  },
	{ L3_AF   , 0x5c, "AF"   , T_100, U_NA  },
	{ L3_ETEMP, 0x5e, "ETEMP", T_INT, U_C   },
	{ L3_DTHR , 0x70, "DTHR" , T_INT, U_PCT },
	{ L3_VBATT, 0x72, "VBATT", T_I1K, U_V   },
	{ L3_VSPED, 0x7a, "VSPED", T_UI , U_KM  },
	{ L3_FUEL1, 0x60, "FUEL1", T_UI , U_US  },
	{ L3_IGNP1, 0x62, "IGNP1", T_INT, U_BD  },
	{ L3_FUEL2, 0x76, "FUEL2", T_UI , U_US  },
	{ L3_IGNP2, 0x78, "IGNP2", T_INT, U_BD  },
	{ L3_ISCV , 0x7e, "ISCV ", T_UI , U_NA  },
	{ L3_ERR1 , 0x6c, "ERR1" , T_UI , U_NA  },
	{ L3_ERR2 , 0x6e, "ERR2" , T_UI , U_NA  }
};
