/*
 *	main.c	HKS FCON-Vpro/S monitor for AKI-PIC standalone
 *			Copyright(C)2003 Kashi Electronics Designs,
 *			All Rights Reserved.
 *			PIC18F452(32MHz) with compiler HiTech-C
 *	Ver 1.0	Sep.8/2003 first release
 *
 *   This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *   This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 */

#include <pic18.h>
#include <stdio.h>
#include <string.h>
#include "delay.h"
#include "lcd.h"
#include "config.h"
#include "fcondb.h"
#include "fconcom.h"
#include "keytimer.h"

//__CONFIG(1, HSPLL);
//__CONFIG(2, BOREN & PWRTEN & WDTDIS);
//__CONFIG(4, DEBUGDIS & LVPDIS & STVREN);

//__CONFIG(4, DEBUGEN & LVPDIS & STVREN);
//__CONFIG(5, CPALL);

void		displabel(char);
void		gettypeval(char, unsigned short *, unsigned char *);
unsigned short	abs2val(char, unsigned short);
unsigned short	val2abs(char, unsigned short);
short	t2step(char);
void		dispvalue(short, char, char);
void		dispbar(char);
void		disppeak(char);
void		dispunit(char);
void		multi_getmon(char);

unsigned char		sbuf[21];
const unsigned char curpos[8] = {
	0x00, 0x40, 0x14,	0x54,
	0x0a, 0x4a, 0x1e,	0x5e
};
const unsigned char bDegC[8] = {
	0x18, 0x18, 0x0e, 0x10,
	0x10, 0x11, 0x0e, 0x00,
};
const unsigned char bPeak[8] = {
	0x18, 0x14, 0x18, 0x15,
	0x17, 0x05, 0x05, 0x00
};

char		lists[8];
unsigned short	warns[4];
unsigned short	peaks[4];
unsigned short	bscale[2];

void	main()
{
	char		i, j, cursel, curmode;
	char		dispreq, entmax;
	char		menuflag;
	unsigned char		type;
	unsigned short	step, val, c;

	// initialize system hardware
	EECON1 = 0;
	TXSTA = 0; RCSTA = 0; SSPCON1 = 0;
	TRISE1 = 0; LATE1 = 1;					// SRAM ~CS1 = high
	TRISC0 = 0; LATC0 = 0;
	TRISE2 = 0; LATE2 = 0;
	LATA0 = 0; LATA1 = 0;					// LED off
	TRISA0 = 0; TRISA1 = 0;
	TRISA2 = 1;
	lcd_init(EIGHTBIT_MODE);
	init_comm();
	init_timer();

	lcd_cmd(0x0c);						// cursor off
	lcd_cmd(2);							// home cursor
	lcd_puts("FCON Monitor Ver1.0");
	lcd_cmd(0x80 | 0x14);
	lcd_puts("     2003(c) Kashi.,");
	lcd_cmd(0x80 | 0x54);
	lcd_puts("All Rights Reserved.");
	for(i=0; i<30;i++) DelayMs(100);


	lcd_cmd(1);							// clear disp.
	for(i = 0; i < 8; i++){				// register bar char to LCD
		lcd_cmd(0x48 + i); lcd_data(0x1b);
		lcd_cmd(0x50 + i); lcd_data(0x18);
		lcd_cmd(0x58 + i); lcd_data(bDegC[i]);
		lcd_cmd(0x60 + i); lcd_data(bPeak[i]);
	}
//
// reload configuration from EEPROM
//
//	0x00 - 0x07	monitor items
//	0x08			interval timer (@25ms)
//	0x09			fcon model
//	0x0a - 0x0d	bar scales
//	0x12 - 0x19	warning levels
//	0x1a			last mode
//
	g_interval = eeprom_read(8);
	g_interval &= 0xfc;
	if((g_interval < 4) || (g_interval > 200)){
		g_interval = 8;
		eeprom_write( 8, 8);
	}

	g_fcontype = eeprom_read(9);
	switch(g_fcontype){
	case FCT_VP31:
		entmax = L1_EOF - 2;
		break;
	case FCT_S:
		entmax = L2_EOF - 2;
		break;
	case FCT_VP30:
		entmax = L1_EOF - 2;
		break;
	case FCT_VP2:
		entmax = L3_EOF - 2;
		break;
	default:
		g_fcontype = FCT_VP31;
		eeprom_write( 9, FCT_VP31);
		entmax = L1_EOF - 2;
		break;
	}	

	for( i = 0; i < 8; i++){
		if( eeprom_read(i) > entmax )
			eeprom_write( i, 0);
	}

	for(i = 0; i < 2; i ++){
		*((char *)bscale + i * 2) = eeprom_read(10 + i * 2);
		*((char *)bscale + i * 2 +1) = eeprom_read(11 + i * 2);
		if(bscale[i] == 0)
			bscale[i] = 1;
	}

	for(i = 0; i < 4; i ++){
		*((char *)warns + i * 2) = eeprom_read(18 + i * 2);
		*((char *)warns + i * 2 +1) = eeprom_read(19 + i * 2);
	}

	curmode = eeprom_read(26);
	if( curmode > 4) curmode = 0;

	GIE = 1;	TMR0ON = 1;							// start interrupt
	i = 0;
	menuflag = 1;

	while(1){
//
// 8 monitor mode
//
		if( curmode == 0){
		  lcd_cmd(0x0c); lcd_cmd(1);
		  lcd_cmd(0x80 | 0x40);
		  lcd_puts("  8 Monitor Mode");
		  lcd_cmd(0x80 | 0x54);
		  lcd_puts("MODE  SEL   UP  DOWN");
		  g_timer1 = 60; menuflag = 1;
		  cursel = 0; LATA1 = 0;

		  while(1){
			if( (menuflag) && (g_timer1 == 0)){
				lcd_cmd(1);						// clear disp.
				for( i = 0; i < 8; i++){
					lists[i] = eeprom_read(i);
					if( lists[i] > entmax ){
						lists[i] = 0;
						eeprom_write( i, 0);
					}
					lcd_cmd(0x80 | curpos[i]);
					displabel(i);
				}
				lcd_cmd(0x80 | curpos[cursel]);
				menuflag = 0;

			} else if(g_key & 0x04){				// SW3, down
				if( lists[cursel] )
					lists[cursel]--;
				g_key = 0;
				if( g_keyRtimer > 20 )
					g_keymask = 8;
				else
					g_keymask = 20;

				displabel(cursel);
				eeprom_write(cursel, lists[cursel]);	// store item to EEPROM
				lcd_cmd(0x80 | curpos[cursel]);

			} else if( g_key & 0x08){				// SW4, up
				if( lists[cursel] < entmax)
					lists[cursel]++;
				g_key = 0;
				if( g_keyRtimer > 20 )
					g_keymask = 8;
				else
					g_keymask = 20;

				displabel(cursel);
				eeprom_write(cursel, lists[cursel]);	// store item to EEPROM
				lcd_cmd(0x80 | curpos[cursel]);

			} else if( g_key & 0x02){				// SW2, sel
				if( cursel < 7 )
					cursel++;
				else
					cursel = 0;
				g_key = 0;
				if( g_keyRtimer > 20 )
					g_keymask = 8;
				else
					g_keymask = 20;

				lcd_cmd(0x80 | curpos[cursel]);
			}

			if(g_key & 0x01){						// SW1, mode
				curmode++;
				g_keymask = 20; g_key = 0;
				eeprom_write(26, curmode);
				break;							// exit this mode
			}

			if( (!menuflag) && (g_loginterval) && (g_keyRtimer == 0)){
				multi_getmon(8);
				lcd_cmd(0x0c);					// cursor off
				if( g_commerr ){
					g_commerr = 0;
					for(j = 0; j < 8; j++){
						lcd_cmd(curpos[j] + 0x85);
						lcd_puts(" --- ");
					}

				} else {
					i = 0;
					for(j = 0; j < 8; j++){
						lcd_cmd(curpos[j] + 0x85);
						gettypeval( lists[j], &val, &type);
						dispvalue( val, type, 5);
						if(j < 4){
							if(val2abs(type, val) >= warns[j])
								i = 1;
						}
					}
					if(i) LATA1 = 1;				// warn LED
					else LATA1 = 0;
				}
				g_loginterval = 0;
				lcd_cmd(0x80 | curpos[cursel]);
				lcd_cmd(0x0f);					// cursor on
			}
		  }
//
// 4 Peak hold monitor mode
//
		} else if( curmode == 1){
		  lcd_cmd(0x0c); lcd_cmd(1);
		  lcd_cmd(0x80 | 0x40);
		  lcd_puts(" 4 PeakHold Mode");
		  lcd_cmd(0x80 | 0x54);
		  lcd_puts("MODE PCLR   UP  DOWN");
		  g_timer1 = 60; menuflag = 1;
		  cursel = 0; LATA1 = 0;
		  while(1){
			if((menuflag) && (g_timer1 == 0)){
				lcd_cmd(1);						// clear disp.
				for( i = 0; i < 4; i++){
					lists[i] = eeprom_read(i);
					if( lists[i] > entmax ){
						lists[i] = 0;
						eeprom_write( i, 0);
					}
					lcd_cmd(0x80 | curpos[i]);
					displabel(i);
				}
				lcd_cmd(curpos[cursel+4] + 0x84);
				for(i = 0; i < 4; i++)
					peaks[i] = 0;
				g_loginterval = 1;
				menuflag = 0;

			} else if(g_key & 0x04){				// SW3, down
				if( lists[cursel] )
					lists[cursel]--;
				g_key = 0;
				if( g_keyRtimer > 20 )
					g_keymask = 8;
				else
					g_keymask = 20;
				lcd_cmd(0x80 | curpos[cursel]);
				displabel(cursel);
				eeprom_write(cursel, lists[cursel]);	// store item to EEPROM
				lcd_cmd(curpos[cursel+4] + 0x84);

			} else if( g_key & 0x08){				// SW4, up
				if( lists[cursel] < entmax)
					lists[cursel]++;
				g_key = 0;
				if( g_keyRtimer > 20 )
					g_keymask = 8;
				else
					g_keymask = 20;

				lcd_cmd(0x80 | curpos[cursel]);
				displabel(cursel);
				eeprom_write(cursel, lists[cursel]);	// store item to EEPROM
				lcd_cmd(curpos[cursel+4] + 0x84);

			} else if( g_key & 0x02){				// SW2, sel, peak clear
				if( cursel < 3 )
					cursel++;
				else
					cursel = 0;
				for(i = 0; i < 4; i++)
					peaks[i] = 0;
				g_loginterval = 1;
				g_key = 0;
				if( g_keyRtimer > 20 )
					g_keymask = 8;
				else
					g_keymask = 20;

				lcd_cmd(curpos[cursel+4] + 0x84);

			}

			if( g_key & 0x01){					// SW1, mode
				curmode++;
				g_keymask = 8; g_key = 0;
				eeprom_write(26, curmode);
				break;							// exit this mode
			}

			if( (!menuflag) && (g_loginterval) && (g_keyRtimer == 0)){
				multi_getmon(4);
				lcd_cmd(0x0c);					// cursor off
				if( g_commerr ){
					g_commerr = 0;
					for(j = 0; j < 4; j++){
						lcd_cmd(curpos[j] + 0x85);
						lcd_puts("   ---");
					}

				} else {
					i = 0;
					for(j = 0; j < 4; j++){
						lcd_cmd(curpos[j] + 0x85);
						gettypeval( lists[j], &val, &type);
						dispvalue( val, type, 6);
						dispunit( lists[j] );
						lcd_cmd(curpos[j + 4] + 0x84);
						disppeak( j );
						if(j < 4){
							if(val2abs(type, val) >= warns[j])
								i = 1;
						}
					}
					if(i) LATA1 = 1;				// warn LED
					else LATA1 = 0;
				}
				g_loginterval = 0;
				lcd_cmd(curpos[cursel+4] + 0x84);
				lcd_cmd(0x0f);					// cursor on
			}

		  }
//
// 2 bar graph mode
//
		} else if( curmode == 2){
		  lcd_cmd(0x0c); lcd_cmd(1);
		  lcd_cmd(0x80 | 0x40);
		  lcd_puts(" 2 BarGraph Mode");
		  lcd_cmd(0x80 | 0x54);
		  lcd_puts("MODE  SEL   UP  DOWN");
		  g_timer1 = 60; menuflag = 1;
		  cursel = 0; LATA1 = 0;

		  while(1){
			if((menuflag) && (g_timer1 == 0)){
				lcd_cmd(1);						// clear disp.
				for( i = 0; i < 2; i++){
					lists[i] = eeprom_read(i);
					if( lists[i] > entmax ){
						lists[i] = 0;
						eeprom_write( i, 0);
					}
					lcd_cmd(0x80 | curpos[i * 2]);
					displabel(i);
				}
				lcd_cmd(curpos[cursel * 2 + 4] + 0x89);
				g_loginterval = 1;
				menuflag = 0;

			} else if(g_key & 0x04){				// SW3, down
				gettypeval( lists[cursel], &val, &type);
				step = t2step(type);

				if(bscale[cursel] < step)
					if( step > 1)
						step >>= 1;

				if(bscale[cursel] > step)
					bscale[cursel] -= step;

				g_key = 0;
				if((g_keyRtimer > 20) || (g_keyRtimer < 120))
					g_keymask = 4;
				else if(g_keyRtimer > 120)
					g_keymask = 10;
				else
					g_keymask = 20;

				menuflag = 1; g_timer1 = 20;
				lcd_cmd(curpos[cursel * 2] + 0x80);
				lcd_puts("Bar Scale ");
				c = abs2val( type, bscale[cursel]);
				dispvalue( c, type, 6);
												// store scales to EEPROM
				eeprom_write(10 + cursel * 2, *((char *)bscale + cursel * 2));
				eeprom_write(11 + cursel * 2, *((char *)bscale + cursel * 2 + 1));
				lcd_cmd(curpos[cursel * 2 + 4] + 0x89);
				
			} else if( g_key & 0x08){				// SW4, up
				gettypeval( lists[cursel], &val, &type);
				step = t2step(type);
				if(bscale[cursel] > (0xffff - step))
					if( step > 1)
						step >>= 1;

				if(bscale[cursel] < (0xffff - step))
					bscale[cursel] += step;

				g_key = 0;
				if((g_keyRtimer > 20) || (g_keyRtimer < 120))
					g_keymask = 4;
				else if(g_keyRtimer > 120)
					g_keymask = 10;
				else
					g_keymask = 20;

				menuflag = 1; g_timer1 = 20;
				lcd_cmd(curpos[cursel * 2] + 0x80);
				lcd_puts("Bar Scale ");
				if(	type == T_CHR || type == T_05C || type == T_NCH )
					if( bscale[cursel] > 255 )
						bscale[cursel] = 255;
				c = abs2val( type, bscale[cursel]);
				dispvalue( c, type, 6);
												// store scales to EEPROM
				eeprom_write(10 + cursel * 2, *((char *)bscale + cursel * 2));
				eeprom_write(11 + cursel * 2, *((char *)bscale + cursel * 2 + 1));
				lcd_cmd(curpos[cursel * 2 + 4] + 0x89);

			} else if( g_key & 0x02){				// SW2, sel
				if( cursel < 1 )
					cursel++;
				else
					cursel = 0;
				g_key = 0;
				if( g_keyRtimer > 20 )
					g_keymask = 8;
				else
					g_keymask = 20;
				lcd_cmd(curpos[cursel * 2 + 4] + 0x89);
			}

			if( g_key & 0x01){					// SW1, mode
				curmode++;
				g_keymask = 8; g_key = 0;
				eeprom_write(26, curmode);
				break;							// exit mode
			}

			if((!menuflag) && (g_loginterval) && (g_keyRtimer == 0)){
				multi_getmon(2);
				lcd_cmd(0x0c);					// cursor off
				if( g_commerr ){
					g_commerr = 0;
					for(j = 0; j < 2; j++){
						lcd_cmd(curpos[j * 2] + 0x85);
						lcd_puts("   ---");
					}

				} else {
					i = 0;
					for(j = 0; j < 2; j++){
						lcd_cmd(curpos[j * 2] + 0x85);
						gettypeval( lists[j], &val, &type);
						dispvalue( val, type, 6);
						dispunit( lists[j] );
						lcd_cmd(curpos[j * 2 + 1] + 0x80);
						dispbar(j);
						if(j < 4){
							if(val2abs(type, val) >= warns[j])
								i = 1;
						}
					}
					if(i) LATA1 = 1;				// warn LED
					else LATA1 = 0;
				}
				g_loginterval = 0;
				lcd_cmd(curpos[cursel * 2 + 4] + 0x89);
				lcd_cmd(0x0f);					// cursor on
			}
		  }
//
// Warnning LED setup mode
//
		} else if( curmode == 3){
		  lcd_cmd(0x0c); lcd_cmd(1);
		  lcd_cmd(0x80 | 0x40);
		  lcd_puts("WarnLED set Mode");
		  lcd_cmd(0x80 | 0x54);
		  lcd_puts("MODE  SEL   UP  DOWN");
		  g_timer1 = 60; menuflag = 1;
		  cursel = 0; LATA1 = 0;

		  while(1){
			if((menuflag) && (g_timer1 == 0)){
				lcd_cmd(1);						// clear disp.
				for( i = 0; i < 4; i++){
					lists[i] = eeprom_read(i);
					if( lists[i] > entmax ){
						lists[i] = 0;
						eeprom_write( i, 0);
					}
					lcd_cmd(curpos[i] + 0x80);
					displabel(i);
					lcd_cmd(curpos[i] + 0x86);
					gettypeval( lists[i], &val, &type);
					c = abs2val( type, warns[i]);
					dispvalue( c, type, 6);
				}
				lcd_cmd(curpos[cursel] + 0x8b);
				g_loginterval = 1;
				menuflag = 0;

			} else if(g_key & 0x04){				// SW3, down
				gettypeval( lists[cursel], &val, &type);
				step = t2step(type);

				if(warns[cursel] < step)
					if( step > 1)
						step >>= 1;

				if(warns[cursel] > step)
					warns[cursel] -= step;

				g_key = 0;
				if((g_keyRtimer > 20) || (g_keyRtimer < 120))
					g_keymask = 4;
				else if(g_keyRtimer > 120)
					g_keymask = 10;
				else
					g_keymask = 20;

				lcd_cmd(curpos[cursel] + 0x86);
				gettypeval( lists[cursel], &val, &type);
				c = abs2val( type, warns[cursel]);
				dispvalue( c, type, 6);
												// store warns to EEPROM
				eeprom_write(18 + cursel * 2, *((char *)warns + cursel * 2));
				eeprom_write(19 + cursel * 2, *((char *)warns + cursel * 2 + 1));
				lcd_cmd(curpos[cursel] + 0x8b);

			} else if( g_key & 0x08){				// SW4, up
				gettypeval( lists[cursel], &val, &type);
				step = t2step(type);
				if(warns[cursel] > (0xffff - step))
					if( step > 1)
						step >>= 1;

				if(warns[cursel] < (0xffff - step))
					warns[cursel] += step;

				g_key = 0;
				if((g_keyRtimer > 20) || (g_keyRtimer < 120))
					g_keymask = 4;
				else if(g_keyRtimer > 120)
					g_keymask = 10;
				else
					g_keymask = 20;

				lcd_cmd(curpos[cursel] + 0x86);
				if(	type == T_CHR || type == T_05C || type == T_NCH )
					if( warns[cursel] > 255 )
						warns[cursel] = 255;
				c = abs2val( type, warns[cursel]);
				dispvalue( c, type, 6);
												// store warns to EEPROM
				eeprom_write(18 + cursel * 2, *((char *)warns + cursel * 2));
				eeprom_write(19 + cursel * 2, *((char *)warns + cursel * 2 + 1));
				lcd_cmd(curpos[cursel] + 0x8b);

			} else if( g_key & 0x02){				// SW2, sel
				if( cursel < 3 )
					cursel++;
				else
					cursel = 0;
				g_key = 0;
				if( g_keyRtimer > 20 )
					g_keymask = 8;
				else
					g_keymask = 20;
				lcd_cmd(curpos[cursel] + 0x8b);
			}

			if( g_key & 0x01){					// SW1, mode
				curmode++;
				g_keymask = 8; g_key = 0;
				eeprom_write(26, curmode);
				break;							// exit mode
			}
		  }
//
// SETUP mode
//
		} else if( curmode == 4){
		  lcd_cmd(0x0c); lcd_cmd(1);
		  lcd_cmd(0x80 | 0x40);
		  lcd_puts("   SETUP    Mode");
		  lcd_cmd(0x80 | 0x54);
		  lcd_puts("MODE  SEL   UP  DOWN");
		  g_timer1 = 60; menuflag = 1;
		  cursel = 0; LATA1 = 0;

		  while(1){
			if( (menuflag) && (g_timer1 == 0)){
				lcd_cmd(1);						// clear disp.
				lcd_puts("SETUP");
				lcd_cmd(0x80 | 0x14);
				lcd_puts("  Interval");
				lcd_cmd(0x80 | 0x54);
				lcd_puts(" FCON type");
				cursel = 0; dispreq = 1;
				lcd_cmd(0x0f);
				menuflag = 0;

			} else if( g_key & 0x04){				// SW3, down
				g_keymask = 8; g_key = 0;
				switch(cursel){
				case 0:
					if( g_interval > 4 )
						g_interval -= 4;
					eeprom_write( 8, g_interval);
					break;
				case 1:
					if( g_fcontype > 0 )
						g_fcontype--;
					eeprom_write( 9, g_fcontype);
					break;
				}
				dispreq = 1;

			} else if( g_key & 0x08){				// SW4, up
				g_keymask = 8; g_key = 0;
				switch(cursel){
				case 0:
					if( g_interval < 200 )
						g_interval += 4;
					eeprom_write( 8, g_interval);
					break;
				case 1:
					if( g_fcontype < (FCT_EOF - 1))
						g_fcontype++;
					eeprom_write( 9, g_fcontype);
					break;
				}
				dispreq = 1;

			} else if( g_key & 0x02){				// SW2, sel
				if( cursel < 1 )
					cursel++;
				else
					cursel = 0;
				g_keymask = 8; g_key = 0;

				switch(cursel){
				case 0:
					lcd_cmd(0x80 | 0x27);
					break;
				case 1:
					lcd_cmd(0x80 | 0x67);
					break;
				}
			}

			if( g_key & 0x01){					// SW1, mode
				curmode = 0;						// last of modes, return top
				g_keymask = 8; g_key = 0;
				eeprom_write(26, curmode);
				break;
			}

			if(dispreq){
				lcd_cmd(0x80 | 0x1e);
				sprintf( sbuf, " %4dmS", (g_interval * 25));
				stralign( sbuf, 10);
				lcd_puts(sbuf);
				lcd_cmd(0x80 | 0x5e);
				switch(g_fcontype){
				case FCT_VP31:
					strcpy( sbuf, "VPRO3.1x");
					entmax = L1_EOF - 1;
					break;
				case FCT_S:
					strcpy( sbuf, "FCON-S");
					entmax = L2_EOF - 1;
					break;
				case FCT_VP30:
					strcpy( sbuf, "VPRO3.0x");
					entmax = L1_EOF - 1;
					break;
				case FCT_VP2:
					strcpy( sbuf, "VPRO2.x");
					entmax = L3_EOF - 1;
					break;
				}
				stralign( sbuf, 10);
				lcd_puts(sbuf);
				switch(cursel){
				case 0:
					lcd_cmd(0x80 | 0x27);
					break;
				case 1:
					lcd_cmd(0x80 | 0x67);
					break;
				}
				dispreq = 0;
			}
			
		  }
		}
	}
}

//
//	display monitor item label
//
void		displabel( char pos )
{	
	char		i, j;

	lcd_cmd(0x0c);				// cursor off
	i = lists[pos];

	switch(g_fcontype){
	case FCT_VP31:
	case FCT_VP30:
		strcpy( sbuf, MonEntVP5[i].label);
		break;
	case FCT_S:
		strcpy( sbuf, MonEntFCS[i].label);
		break;
	case FCT_VP2:
		strcpy( sbuf, MonEntVP2[i].label);
		break;
	}

	j = strlen(sbuf);
	sbuf[5] = 0; i = 4;
	while(j <= 4)
		sbuf[j++] = 0x20;
	lcd_puts(sbuf);
	lcd_cmd(0x0f);				// cursor on

}

//
//	get value by items from receive buffer
//
void		gettypeval( char k, unsigned short *val, unsigned char *t)
{
	unsigned char		type, addr;
	unsigned short	v;

	switch(g_fcontype){
	case FCT_VP31:
	case FCT_VP30:
		type = MonEntVP5[k].type;
		addr = MonEntVP5[k].addr;
		break;
	case FCT_S:
		type = MonEntFCS[k].type;
		addr = MonEntFCS[k].addr;
		break;
	case FCT_VP2:
		type = MonEntVP2[k].type;
		addr = MonEntVP2[k].addr;
		break;
	}

	switch( type ){
	case T_INT:
	case T_TIN:
	case T_DEG:
	case T_35I:
	case T_100:
	case T_UI:
	case T_IM5:
	case T_I1K:
	default:
		v = fcon_GetMonValW(addr);
		break;

	case T_CHR:
	case T_NCH:
	case T_05C:
	case T_BOL:
		v = (unsigned short)fcon_GetMonVal(addr);
		break;
	}

	*val = v; *t = type;
}

//
//	display value with unit convert
//
void		dispvalue(short v, char type, char w)
{
	switch( type ){
	case T_INT:
	case T_TIN:
	default:
		int2str( sbuf, v);
		break;
	case	T_CHR:
		char2str( sbuf, v);
		break;
	case	T_NCH:
		nchar2str( sbuf, (signed char)v);
		break;
	case T_BOL:
		bool2str( sbuf, v);
		break;
	case	T_DEG:
		deg2str( sbuf, v);
		break;
	case T_35I:
		i352str( sbuf, v);
		break;
	case T_100:
		cent2str( sbuf, v);
		break;
	case T_05C:
		c052str( sbuf, v);
		break;
	case T_UI:
		uint2str( sbuf, v);
		break;
	case T_IM5:
		im52str( sbuf, v);
		break;
	case T_I1K:
		i1k2str( sbuf, v);
		break;
	}
	stralign( sbuf, w);
	sbuf[w] = 0;
	lcd_puts(sbuf);
}

//
//	display bar graph by monitor lists
//
void		dispbar( char pos )
{
	unsigned char		i, j, k;
	unsigned short	v, c;
	unsigned long		l;

	k = lists[pos];
	gettypeval( k, &v, &i);
	c = val2abs( i, v);

	l = (c * (unsigned long)40) / bscale[pos];
	j = (unsigned char)l;
	for(i = 0; i < 20; i++)
		sbuf[i] = 0x20;

	for( i = 0; i < 40; i++){
		if( i == j ){
			if(j % 2)
				sbuf[i / 2] = 2;
			break;
		}
		sbuf[i / 2] = 1;
	}
	sbuf[20] = 0;
//	sprintf(sbuf, "%d/%d", c, bscale[pos]);
	lcd_puts(sbuf);
}

//
//	display peak value
//
void		disppeak( char pos )
{
	unsigned char		type, k;
	unsigned short	v, c;

	k = lists[pos];
	gettypeval( k, &v, &type);
	c = val2abs( type, v);
	if( c >= peaks[pos] )
		peaks[pos] = c;
	else
		return;

	lcd_data(4);					// p/h char
	gettypeval( k, &v, &type);
	dispvalue( v, type, 5);
}

//
//	display unit string
//
void		dispunit( char ent )
{
	unsigned char		unit;

	switch(g_fcontype){
	case FCT_VP31:
	case FCT_VP30:
		unit = MonEntVP5[ent].unit;
		break;
	case FCT_S:
		unit = MonEntFCS[ent].unit;
		break;
	case FCT_VP2:
		unit = MonEntVP2[ent].unit;
		break;
	}

	lcd_puts(sUnit[unit]);
}

//
//	convert real value from absolute scale value
//
unsigned short	abs2val( char type, unsigned short vabs)
{
	unsigned short	v;
	char		c;

	switch( type ){
	case T_INT:
		v = (short)(vabs - 0x8000);
		break;

	case T_NCH:
		v = - (vabs - 127);
		c = (signed char)v;
		v = c;
		break;
	default:
		v = vabs;
	}

	return	v;
}

//
//	convert absolute scale value from real value
//
unsigned short	val2abs( char type, unsigned short val)
{
	unsigned short	v;
	char		c;

	switch( type ){
	case T_INT:
		v = (unsigned short)val + 0x8000;
		break;

	case T_NCH:
		c = val;
		v = (signed char)-c;
		v += 127;
		break;
	default:
		v = val;
	}

	return	v;
}

//
//	get stepping value from item
//
short	t2step( char type)
{
	short	step;

	switch( type ){
	case T_TIN:
	case T_CHR:
	case T_NCH:
	case T_05C:
		step = 1;
		if( g_keyRtimer >= 120 )
			step = 20;
		break;
	case T_100:
		step = 1;
		if( g_keyRtimer >= 120 )
			step = 10;
		break;
	case T_I1K:
		step = 10;
		if( g_keyRtimer >= 120 )
			step = 100;
		break;
	case T_35I:
		step = 8;
		if( g_keyRtimer >= 120 )
			step = 80;
		break;
	default:
		step = 1;
		if( g_keyRtimer >= 120 )
			step = 256;
		break;
	}

	return	step;
}

//
//	FCON data exchange with minimized transceive
//
void		multi_getmon(char emax)
{
	unsigned char		i, j, mask;

	mask = 0;
	for( i = 0; i < emax; i++){
		switch(g_fcontype){
		case FCT_VP31:
		case FCT_VP30:
			j = MonEntVP5[ lists[i] ].addr & 0xf0;
			break;
		case FCT_S:
			j = MonEntFCS[ lists[i] ].addr & 0xf0;
			break;
		case FCT_VP2:
			j = MonEntVP2[ lists[i] ].addr & 0xf0;
			break;
		}
		j = j >> 4;
		mask |= 1 << j;
	}
	for( i = 0; i < 8; i++){
		if( mask & ( 1 << i )){
			j = i << 4;
			fcon_getmon16(j);
			if(g_commerr){
				LATA0 = 0;		// off green LED
				return;
			}
		}
	}
	LATA0 = 1;					// green LED
	g_comled = 40;				// off decay 1000ms
}

